@@ -32,7 +32,7 @@
 
 #include "config.h"
 
-#include <stddef.h>
+#include "sysincl.h"
 
 #include "manual.h"
 #include "logging.h"
@@ -93,8 +93,6 @@ MNL_Initialise(void)
   error = ERROR_MARGIN;
 
   LCL_AddParameterChangeHandler(slew_samples, NULL);
-
-  return;
 }
 
 /* ================================================== */
@@ -102,7 +100,6 @@ MNL_Initialise(void)
 void
 MNL_Finalise(void)
 {
-  return;
 }
 
 /* ================================================== */
@@ -157,7 +154,7 @@ estimate_and_set_system(struct timeval *now, int offset_provided, double offset,
   
   if (found_freq) {
     LOG(LOGS_INFO, LOGF_Manual,
-        "Making a frequency change of %.3fppm and a slew of %.6f\n",
+        "Making a frequency change of %.3f ppm and a slew of %.6f",
         1.0e6 * freq, slew_by);
     
     REF_SetManualReference(now,
@@ -239,7 +236,6 @@ slew_samples(struct timeval *raw,
         dfreq, doffset);
     samples[i].offset += delta_time;
   }
-  return;
 }
 
 /* ================================================== */
