@@ -1,8 +1,4 @@
 /*
-  $Header: /cvs/src/chrony/nameserv.h,v 1.8 2002/02/28 23:27:11 richard Exp $
-
-  =======================================================================
-
   chronyd/chronyc - Programs for keeping computer clocks accurate.
 
  **********************************************************************
@@ -34,12 +30,20 @@
 
 #include "addressing.h"
 
+typedef enum {
+  DNS_Success,
+  DNS_TryAgain,
+  DNS_Failure
+} DNS_Status;
+
 /* Resolve names only to selected address family */
 extern void DNS_SetAddressFamily(int family);
 
-extern int DNS_Name2IPAddress(const char *name, IPAddr *addr, int retry);
+extern DNS_Status DNS_Name2IPAddress(const char *name, IPAddr *addr);
 
 extern int DNS_IPAddress2Name(IPAddr *ip_addr, char *name, int len);
 
+extern void DNS_Reload(void);
+
 #endif /* GOT_NAMESERV_H */
 
