@@ -114,6 +114,11 @@ RCL_Initialise(void)
 {
   CNF_AddRefclocks();
 
+  if (n_sources > 0) {
+    LCL_AddParameterChangeHandler(slew_samples, NULL);
+    LCL_AddDispersionNotifyHandler(add_dispersion, NULL);
+  }
+
   logfileid = CNF_GetLogRefclocks() ? LOG_FileOpen("refclocks",
       "   Date (UTC) Time         Refid  DP L P  Raw offset   Cooked offset      Disp.")
     : -1;
@@ -268,11 +273,6 @@ RCL_StartRefclocks(void)
     } else
       inst->lock_ref = -1;
   }
-
-  if (n_sources > 0) {
-    LCL_AddParameterChangeHandler(slew_samples, NULL);
-    LCL_AddDispersionNotifyHandler(add_dispersion, NULL);
-  }
 }
 
 void
@@ -452,7 +452,7 @@ RCL_AddPulse(RCL_Instance instance, struct timeval *pulse_time, double second)
   filter_add_sample(&instance->filter, &cooked_time, offset, dispersion);
   instance->leap_status = LEAP_Normal;
 
-  log_sample(instance, &cooked_time, 0, 1, second, offset, dispersion);
+  log_sample(instance, &cooked_time, 0, 1, offset + correction - instance->offset, offset, dispersion);
 
   /* for logging purposes */
   if (!instance->driver->poll)
